-- ====================================================================
-- IMG_MOVER_VIEWER.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Company: MB
-- ====================================================================
-- Revision:   ver x.x - date xx gen 2007
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity img_mover_viewer is
   port(
      -- system interface
      sys_rst        : in std_logic;
      tft_clk        : in std_logic;
      -- micro_emulator interface
      img_start_move : in std_logic; -- attivo alto un solo tft_clk
      img_view       : in std_logic; -- attivo alto un solo tft_clk
      img_nr         : in std_logic_vector(3 downto 0);
      img_end_move   : out std_logic;
      -- pixel_addr_data_fifo interface
      fifo_wr_req    : out std_logic;
      fifo_wr_add    : out std_logic_vector(21 downto 0);
      fifo_wr_rgb    : out std_logic_vector(15 downto 0);
		-- tft_timing_ctrl interface
		tft_bank       : out std_logic_vector(1 downto 0);
		tft_sect       : out std_logic_vector(2 downto 0);
      -- spi_flash_reader interface
      spi_start_rd   : out std_logic;
      spi_addr       : out std_logic_vector(7 downto 0);
      spi_byte_nr    : out std_logic_vector(20 downto 0);
      spi_byte_en    : in std_logic;
      spi_byte       : in std_logic_vector(7 downto 0);
      spi_end_rd     : in std_logic
   );
end img_mover_viewer;
-----------------------------------------------------------------------
architecture rtl_img_mover_viewer of img_mover_viewer is

	constant PIXEL_ADDR_0 : std_logic_vector(18 downto 0) := (others => '0'); -- 19 bit

   -- con output register !!!
   component img_spi_sector_rom
      port (
         Address    : in std_logic_vector(2 downto 0); 
         OutClock   : in std_logic; 
         OutClockEn : in std_logic; 
         Reset      : in std_logic; 
         Q          : out std_logic_vector(7 downto 0)
      );
   end component;

   type move_view_state is (IDLE, SPI_START_ST, WR_ADDR_ST, WR_READ_ST,
	                         WAIT_B_DATA_ST, WR_B_DATA_ST,
	                         WAIT_G_DATA_ST, WR_G_DATA_ST,
	                         WAIT_R_DATA_ST, WR_R_DATA_ST,
									 WR_PIXEL_FIFO_ST, WAIT_PIXEL_FIFO_ST);

   signal move_view_st : move_view_state;

   signal spi_sector_addr : std_logic_vector(7 downto 0);

	signal addr_data_wr : std_logic;
   signal pixel_addr   : std_logic_vector(21 downto 0);
   signal pixel_data   : std_logic_vector(15 downto 0);

	signal addr_data_wr_reg : std_logic;
   signal pixel_addr_reg   : std_logic_vector(21 downto 0);
   signal pixel_data_reg   : std_logic_vector(15 downto 0);

begin

   IMG_SPI_SECTOR_ROM_INST: img_spi_sector_rom
      port map(
         Address    => img_nr(2 downto 0),
         OutClock   => tft_clk,
         OutClockEn => img_start_move,
         Reset      => sys_rst,
         Q          => spi_sector_addr
      );

   -- indirizzo del settore della SPI FLASH
   spi_addr <= spi_sector_addr;

   ----------
   -- 800x600
   ----------
   -- 800x600 pixel = 480000 pixel x 3 byte/pixel - 1 = 1439999 = 101011111100011111111
   --spi_byte_nr <= "101011111100011111111";
   -- 800x600 pixel = 480000 pixel x 2 byte/pixel - 1 = 959999 = 11101010010111111111
   -- originale che non copia la parte tra le immagini spi_byte_nr <= "011101010010111111111"; -- 011101010010111111111 bin =  959999 dec = EA5FF 
   -- che equivale a EA600 -1
   -- 0xF0000 - 1 = 0xEFFFF = 11101111111111111111
   spi_byte_nr <= "011101111111111111111";
               
   ----------
   -- 320x240
   ----------
   -- 320x240 pixel = 76800 pixel x 3 byte/pixel - 1 = 230339 = 111000001111111111
   --spi_byte_nr <= "00111000001111111111";
   ----------
   -- 480x272
   ----------
   -- 480x272 pixel = 130560 pixel x 3 byte/pixel - 1 = 391679 = 111000001111111111
   --spi_byte_nr <= "01011111100111111111";
   --------------
   -- simulazione
   --------------
   -- 768 pixel x 3 byte/pixel - 1 = 2303 = 100011111111
   --spi_byte_nr <= "00000000100011111111";
   --spi_byte_nr <= "00000000000011111111";

   ----------------------------------------------------------------------
   -- IMG_MOVER_VIEWER FSM
   -- 1) copia immagine da SPI FLASH a SDRAM
   -- 2) set registri CFG per visualizzazione di immagine da SDRAM su TFT
   ----------------------------------------------------------------------
   IMG_MOVER_VIEWER_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         move_view_st <= IDLE;
         spi_start_rd <= '0';
         img_end_move <= '0';
			--
			addr_data_wr <= '0';
         pixel_addr   <= (others => '0');
         pixel_data   <= (others => '0');
			tft_bank     <= (others => '0');
			tft_sect     <= (others => '0');
			--
      elsif (tft_clk = '1' and tft_clk'event) then
         -- default
         spi_start_rd <= '0';
         img_end_move <= '0';
			--
			addr_data_wr <= '0';
			--
         case move_view_st is
            when IDLE =>
               if (img_start_move = '1') then
                  move_view_st <= SPI_START_ST;
                  spi_start_rd <= '1';
               elsif (img_view = '1') then
                  move_view_st <= WR_READ_ST;
						tft_bank     <= img_nr(2 downto 1);
						tft_sect     <= ("00" & img_nr(0));
               end if;
            when WR_READ_ST =>
               move_view_st <= IDLE;
            when SPI_START_ST =>
               move_view_st <= WR_ADDR_ST;
					pixel_addr   <= (img_nr(2 downto 0) & PIXEL_ADDR_0);
            when WR_ADDR_ST =>
               move_view_st <= WAIT_B_DATA_ST;
            when WAIT_B_DATA_ST =>
               if (spi_end_rd = '1') then
                  move_view_st <= IDLE;
                  img_end_move <= '1';
               elsif (spi_byte_en = '1') then
                  move_view_st <= WR_B_DATA_ST;
                  --pixel_data(4 downto 0)  <= spi_byte(7 downto 3);
                  --pixel_data(10 downto 8) <= spi_byte(2 downto 0);

						-- pixel_data(4) <= spi_byte(3);
						-- pixel_data(3) <= spi_byte(4);
						-- pixel_data(2) <= spi_byte(5);
						-- pixel_data(1) <= spi_byte(6);
						-- pixel_data(0) <= spi_byte(7);

						-- pixel_data(7) <= spi_byte(0);
						-- pixel_data(6) <= spi_byte(1);
						-- pixel_data(5) <= spi_byte(2);

						pixel_data(4) <= spi_byte(4);
						pixel_data(3) <= spi_byte(3);
						pixel_data(2) <= spi_byte(2);
						pixel_data(1) <= spi_byte(1);
						pixel_data(0) <= spi_byte(0);

						pixel_data(7) <= spi_byte(7);
						pixel_data(6) <= spi_byte(6);
						pixel_data(5) <= spi_byte(5);

                  --pixel_data(15 downto 8) <= spi_byte(7 downto 0);
               end if;
            when WR_B_DATA_ST =>
               move_view_st <= WAIT_R_DATA_ST;
            -- when WAIT_G_DATA_ST =>
               -- if (spi_end_rd = '1') then
                  -- move_view_st <= IDLE;
                  -- img_end_move <= '1';
               -- elsif (spi_byte_en = '1') then
                  -- move_view_st <= WR_G_DATA_ST;
                  -- pixel_data(10 downto 5) <= spi_byte(7 downto 2);
               -- end if;
            -- when WR_G_DATA_ST =>
               -- move_view_st <= WAIT_R_DATA_ST;
            when WAIT_R_DATA_ST =>
               if (spi_end_rd = '1') then
                  move_view_st <= IDLE;
                  img_end_move <= '1';
               elsif (spi_byte_en = '1') then
                  move_view_st <= WR_R_DATA_ST;
                  --pixel_data(15 downto 11)  <= spi_byte(4 downto 0);
                  --pixel_data(7 downto 5)    <= spi_byte(7 downto 5);

						pixel_data(15) <= spi_byte(7);
						pixel_data(14) <= spi_byte(6);
						pixel_data(13) <= spi_byte(5);
						pixel_data(12) <= spi_byte(4);
						pixel_data(11) <= spi_byte(3);

						pixel_data(10) <= spi_byte(2);
						pixel_data(9) <= spi_byte(1);
						pixel_data(8) <= spi_byte(0);

                  --pixel_data(7 downto 0) <= spi_byte(7 downto 0);
               end if;
            when WR_R_DATA_ST =>
               move_view_st <= WR_PIXEL_FIFO_ST;
					addr_data_wr <= '1';
            when WR_PIXEL_FIFO_ST =>
               move_view_st <= WAIT_PIXEL_FIFO_ST;
            when WAIT_PIXEL_FIFO_ST =>
               move_view_st <= WAIT_B_DATA_ST;
					pixel_addr   <= pixel_addr + 1;
				when others =>
               move_view_st <= IDLE;
               spi_start_rd <= '0';
               img_end_move <= '0';
					addr_data_wr <= '0';
					pixel_addr   <= (others => '0');
					pixel_data   <= (others => '0');
					tft_bank     <= (others => '0');
					tft_sect     <= (others => '0');
         end case;
      end if;
   end process img_mover_viewer_FSM_PROC;

   --------------------------------------
   -- PIXEL_ADDR_DATA_WR_REG_PROC process
   --------------------------------------
   PIXEL_ADDR_DATA_WR_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			addr_data_wr_reg <= '0';
			pixel_addr_reg   <= (others => '0');
			pixel_data_reg   <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			addr_data_wr_reg <= addr_data_wr;
			pixel_addr_reg   <= pixel_addr;
			pixel_data_reg   <= pixel_data;
		end if;
   end process PIXEL_ADDR_DATA_WR_REG_PROC;

	fifo_wr_req <= addr_data_wr_reg;
	fifo_wr_add <= pixel_addr_reg;
	fifo_wr_rgb <= pixel_data_reg;

end rtl_img_mover_viewer;
-----------------------------------------------------------------------