-- ====================================================================
-- SPI_FLASH_READER.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Company: MB
-- ====================================================================
-- Revision:   ver x.x - date xx gen 2007
-- ====================================================================
-- Note:
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity spi_flash_reader is
   port(
      -- system interface
      sys_rst      : in std_logic;
      spi_clk_in   : in std_logic;
      -- spi flash interface
      spi_csn      : out std_logic;
      spi_clk      : out std_logic;
      spi_d        : out std_logic;
      spi_q        : in std_logic;
      -- micro_emulator interface
      spi_start_rd : in std_logic;
      spi_addr     : in std_logic_vector(7 downto 0);
      spi_byte_nr  : in std_logic_vector(20 downto 0);
      spi_byte_en  : out std_logic;
      spi_byte     : out std_logic_vector(7 downto 0);
      spi_end_rd   : out std_logic
   );
end spi_flash_reader;
-----------------------------------------------------------------------
architecture rtl_spi_flash_reader of spi_flash_reader is

   type spi_rd_state is (IDLE, SPI_CS_ST, SPI_CMD_ADDR_ST,
                         SPI_DATA_ST, SPI_END_ST);
   
   signal spi_rd_st : spi_rd_state;

   signal spi_clk_en : std_logic;

   signal spi_d_sr_en : std_logic;
   signal spi_q_sr_en : std_logic;
   
   signal rst_d_cnt : std_logic;
   signal rst_q_cnt : std_logic;

   signal d_cnt : std_logic_vector(4 downto 0);
   signal q_cnt : std_logic_vector(2 downto 0);

   signal spi_d_sr : std_logic_vector(31 downto 0);
   signal spi_q_sr : std_logic_vector(7 downto 0);

   signal spi_byte_cnt : std_logic_vector(20 downto 0);

   signal spi_finish : std_logic;

   signal spi_end_rd_reg1 : std_logic;
   signal spi_end_rd_reg2 : std_logic;
   signal spi_end_rd_reg3 : std_logic;

   signal rgb_byte_en : std_logic;

   signal rgb_byte : std_logic_vector(7 downto 0);

begin

   ---------------------------------------
   -- Shift register SPI command e address
   ---------------------------------------
   SPI_D_SR_PROC: process(sys_rst, spi_clk_in)
   begin
      if (sys_rst = '1') then
         spi_d_sr <= (others => '0');
      elsif (spi_clk_in = '0' and spi_clk_in'event) then -- NB: fronte di discesa
         if (spi_start_rd = '1') then
            spi_d_sr <= ("00000011" & spi_addr & "00000000" & "00000000");
         elsif (spi_d_sr_en = '1') then
            spi_d_sr(31 downto 1) <= spi_d_sr(30 downto 0);
         end if;
      end if;
   end process SPI_D_SR_PROC;

   -- SPI FLASH data output
   spi_d <= spi_d_sr(31) when (spi_d_sr_en = '1') else
            '1';

   -- SPI FLASH clock output
   -- spi_clk <= spi_clk_in when (spi_clk_en = '1') else
              -- 'Z'; -- pull-down esterno alla EC
   spi_clk <= spi_clk_in when (spi_clk_en = '1') else
              '0'; -- pull-down esterno alla EC

   --------------------------------------
   -- Shift register SPI data dalla flash
   --------------------------------------
   SPI_Q_SR_PROC: process(sys_rst, spi_clk_in)
   begin
      if (sys_rst = '1') then
         spi_q_sr <= (others => '0');
      elsif (spi_clk_in = '1' and spi_clk_in'event) then -- NB: fronte di salita
         if (spi_q_sr_en = '1') then
            spi_q_sr(0)          <= spi_q;
            spi_q_sr(7 downto 1) <= spi_q_sr(6 downto 0);
         end if;
      end if;
   end process SPI_Q_SR_PROC;

   ------------
   -- D counter
   ------------
   D_CNT_PROC: process(spi_clk_in)
   begin
      if (spi_clk_in = '0' and spi_clk_in'event) then -- NB: fronte di discesa
         if (rst_d_cnt = '1') then
            d_cnt <= (others => '0');
         else
            d_cnt <= d_cnt + 1;
         end if;
      end if;
   end process D_CNT_PROC;

   ------------
   -- Q counter
   ------------
   Q_CNT_PROC: process(spi_clk_in)
   begin
      if (spi_clk_in = '0' and spi_clk_in'event) then -- NB: fronte di discesa
         if (rst_q_cnt = '1') then
            q_cnt <= (others => '0');
         else
            q_cnt <= q_cnt + 1;
         end if;
      end if;
   end process Q_CNT_PROC;

   -----------------------
   -- SPI_BYTE_CNT counter
   -----------------------
   SPI_BYTE_CNT_PROC: process(spi_clk_in)
   begin
      if (spi_clk_in = '0' and spi_clk_in'event) then -- NB: fronte di discesa
         if (spi_start_rd = '1') then
            spi_byte_cnt <= spi_byte_nr;
         elsif (rgb_byte_en = '1') then
            spi_byte_cnt <= spi_byte_cnt - 1;
         end if;
      end if;
   end process SPI_BYTE_CNT_PROC;

   ----------------------------------
   -- FSM per lettura della SPI FLASH
   ----------------------------------
   SPI_FLASH_READER_FSM_PROC: process(sys_rst, spi_clk_in)
   begin
      if (sys_rst = '1') then
         spi_rd_st   <= IDLE;
         spi_csn     <= '1';
         spi_clk_en  <= '0';
         spi_d_sr_en <= '0';
         spi_q_sr_en <= '0';
         rst_d_cnt   <= '1';
         rst_q_cnt   <= '1';
         rgb_byte_en <= '0';
         spi_finish  <= '0';
         rgb_byte    <= (others => '0');
      elsif (spi_clk_in = '0' and spi_clk_in'event) then -- NB: fronte di discesa
         -- default
         rst_d_cnt   <= '0';
         rst_q_cnt   <= '0';
         rgb_byte_en <= '0';
         spi_finish  <= '0';
         case spi_rd_st is
            when IDLE =>
               if (spi_start_rd = '1') then
                  spi_rd_st   <= SPI_CS_ST;
                  spi_csn     <= '0';
                  rst_d_cnt   <= '1';
                  rst_q_cnt   <= '1';
               end if;
            when SPI_CS_ST =>
               spi_rd_st   <= SPI_CMD_ADDR_ST;
               spi_clk_en  <= '1';
               spi_d_sr_en <= '1';
            when SPI_CMD_ADDR_ST =>
               if (d_cnt = 31) then
                  spi_rd_st   <= SPI_DATA_ST;
                  spi_d_sr_en <= '0';
                  spi_q_sr_en <= '1';
               end if;
            when SPI_DATA_ST =>
               if (q_cnt = 7) then
                  rgb_byte_en <= '1';
                  rgb_byte    <= spi_q_sr;
                  if (spi_byte_cnt = 0) then
                     spi_rd_st   <= SPI_END_ST;
                     spi_csn     <= '1';
                     spi_clk_en  <= '0';
                     spi_d_sr_en <= '0';
                     spi_q_sr_en <= '0';
                  end if;
               end if;
            when SPI_END_ST =>
               spi_rd_st  <= IDLE;
               spi_finish <= '1';
            when others =>
               spi_rd_st   <= IDLE;
               spi_csn     <= '1';
               spi_clk_en  <= '0';
               spi_d_sr_en <= '0';
               spi_q_sr_en <= '0';
               rst_d_cnt   <= '1';
               rst_q_cnt   <= '1';
               spi_finish  <= '0';
         end case;
      end if;
   end process SPI_FLASH_READER_FSM_PROC;

   ---------------------------------------------------------------------------
   -- Registrazione di RGB_BYTE ed RGB_BYTE_EN sul fronte di salita SPI_CLK_IN
   ---------------------------------------------------------------------------
   BYTE_REG_PROC: process(sys_rst, spi_clk_in)
   begin
      if (sys_rst = '1') then
         spi_end_rd_reg1 <= '0';
         spi_end_rd_reg2 <= '0';
         spi_end_rd_reg3 <= '0';
         spi_end_rd  <= '0';
         spi_byte_en <= '0';
         spi_byte    <= (others => '0');
      elsif (spi_clk_in = '1' and spi_clk_in'event) then -- NB: fronte di salita
         spi_end_rd_reg1 <= spi_finish;
         spi_end_rd_reg2 <= spi_end_rd_reg1;
         spi_end_rd_reg3 <= spi_end_rd_reg2;
         spi_end_rd      <= spi_end_rd_reg3;
         spi_byte_en <= rgb_byte_en;
         spi_byte    <= rgb_byte;
      end if;
   end process BYTE_REG_PROC;

end rtl_spi_flash_reader;
-----------------------------------------------------------------------