-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- POWER_MANAGER.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
--
-- OSCH will be used as timer with 2.08 MHz frequency
--
-- TFTSurfer POWER-ON:
-- 1) PLL Lock-in Time = 15 ms max
-- 2) SDRAM, after VCC and stable clock,
--    requires init cycle after 100 us
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
entity power_manager is
   port(
      sys_rst     : in std_logic;
      sys_clk     : in std_logic;
      tft_clk     : in std_logic;
		osch_clk    : in std_logic;
      pll_lock    : in std_logic;
      tft_off     : in std_logic;
      tft_on      : in std_logic;
      tft_vtick   : in std_logic;
      delay_100us : out std_logic;
      tft_vdd_en  : out std_logic;
      tft_avdd_en : out std_logic; -- SH43
      tft_disp_en : out std_logic; -- SH43
      tft_sig_en  : out std_logic;
      tft_rgb_en  : out std_logic;
      tft_led_en  : out std_logic
   );
end power_manager;
-----------------------------------------------------------------------
architecture rtl_power_manager of power_manager is

   -----------------------------------------------------------
	-- FREQUENCY osch_clk = 2.08 MHz -> PERIOD sys_clk = 500 ns
	--  15 ms / 500 ns = 30.000 -> 15 bit counter
	-- 100 us / 500 ns = 200    -> 8 bit counter
   -----------------------------------------------------------
   constant CNT_100US_UPPER_BIT : natural := 14;

   ------------------------
	-- functional simulation
   ------------------------
   -- constant CONSTANT_DLY_PLL_15MS    : natural := 31;
   -- constant CONSTANT_DLY_SDRAM_100US : natural := 15;
   ------------------------
   -- functional simulation
   ------------------------

   --constant CONSTANT_DLY_PLL_15MS    : natural := 32767; -- 150 us/180 ns = 833
   --constant CONSTANT_DLY_SDRAM_100US : natural := 255;  -- 100 us/180 ns = 555	
   -- simulazione 
   constant CONSTANT_DLY_PLL_15MS    : natural := 20; -- 150 us/180 ns = 833
   constant CONSTANT_DLY_SDRAM_100US : natural := 5;  -- 100 us/180 ns = 555

   signal timer_cnt : std_logic_vector(CNT_100US_UPPER_BIT downto 0);

   signal timer_cnt_rst : std_logic;
   signal delay_100_us : std_logic;

   signal pll_lock_reg0 : std_logic;
   signal pll_lock_reg1 : std_logic;

   signal delay_100us_reg0 : std_logic;
   signal delay_100us_reg1 : std_logic;

   type delay_fsm_state is (IDLE, RST_15MS, WAIT_15MS, WAIT_LOCK,
                            RST_100US, WAIT_100US, READY);

   signal delay_st : delay_fsm_state;

   signal timer_cnt_15ms_sig : std_logic;
   signal timer_cnt_15ms_reg : std_logic;
   signal timer_cnt_100us_sig : std_logic;
   signal timer_cnt_100us_reg : std_logic;

   type tft_on_off_state is (IDLE, ON_VDD, ON_AVDD, ON_SIG_RGB, ON_DISP,
                             OFF_DISP, OFF_SIG_RGB, OFF_AVDD, OFF_VDD);

   signal tft_on_off_st : tft_on_off_state;

   signal tft_off_reg1 : std_logic;
   signal tft_off_reg2 : std_logic;
   signal tft_on_reg1  : std_logic;
   signal tft_on_reg2  : std_logic;

   signal frame_cnt_rst : std_logic;

   signal frame_cnt : std_logic_vector(3 downto 0);

   signal frame_cnt_3_sig   : std_logic;
   signal frame_cnt_3_reg   : std_logic;
   signal frame_cnt_7_sig   : std_logic;
   signal frame_cnt_7_reg   : std_logic;
   signal frame_cnt_15_sig  : std_logic;
   signal frame_cnt_15_reg  : std_logic;

begin

   ----------------------------
   -- PLL_LOCK_REG_PROC process
   ----------------------------
   PLL_LOCK_REG_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         pll_lock_reg0 <= '0';
         pll_lock_reg1 <= '0';
      elsif (osch_clk = '1' and osch_clk'event) then
         pll_lock_reg0 <= pll_lock;
         pll_lock_reg1 <= pll_lock_reg0;
      end if;
   end process PLL_LOCK_REG_PROC;

   ----------------------------------------------
   -- DELAY FSM per gestione ritardi PLL ed SDRAM
   ----------------------------------------------
   DELAY_FSM_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         delay_st      <= IDLE;
         delay_100_us  <= '0';
         timer_cnt_rst <= '1';
      elsif (osch_clk = '1' and osch_clk'event) then
         case delay_st is
            when IDLE =>
               delay_st      <= RST_15MS;
               delay_100_us  <= '0';
               timer_cnt_rst <= '1';
            when RST_15MS =>
               delay_st      <= WAIT_15MS;
               delay_100_us  <= '0';
               timer_cnt_rst <= '0';
            when WAIT_15MS =>
               timer_cnt_rst <= '0';
               delay_100_us  <= '0';
               if (timer_cnt_15ms_reg = '1') then
                  delay_st <= WAIT_LOCK;
               end if;
            when WAIT_LOCK =>
               delay_100_us <= '0';
               if (pll_lock_reg1 = '1') then
                  delay_st      <= RST_100US;
                  timer_cnt_rst <= '1';
               end if;
            when RST_100US =>
               delay_st      <= WAIT_100US;
               delay_100_us  <= '0';
               timer_cnt_rst <= '0';
            when WAIT_100US =>
               timer_cnt_rst <= '0';
               if (timer_cnt_100us_reg = '1') then
                  delay_st     <= READY;
                  delay_100_us <= '1';
               end if;
            when READY =>
               delay_st     <= READY;
               timer_cnt_rst <= '0';
               delay_100_us <= '1';
            when others =>
               delay_st     <= IDLE;
               timer_cnt_rst <= '1';
               delay_100_us <= '0';
         end case;
      end if;
   end process DELAY_FSM_PROC;

   -------------------------
   -- TIMER_CNT_PROC process
   -------------------------
   TIMER_CNT_PROC: process(osch_clk)
   begin
      if (osch_clk = '1' and osch_clk'event) then
         if (timer_cnt_rst = '1') then
            timer_cnt <= (others => '0');
         else
            timer_cnt <= timer_cnt + 1;
         end if;
      end if;
   end process TIMER_CNT_PROC;
  
   timer_cnt_15ms_sig <= '1' when (timer_cnt = CONSTANT_DLY_PLL_15MS) else
                         '0';

   timer_cnt_100us_sig <= '1' when (timer_cnt = CONSTANT_DLY_SDRAM_100US) else
                          '0';

   -----------------------------
   -- TIMER_CNT_REG_PROC process
   -----------------------------
   TIMER_CNT_REG_PROC: process(sys_rst, osch_clk)
   begin
      if (sys_rst = '1') then
         timer_cnt_15ms_reg  <= '0';
         timer_cnt_100us_reg <= '0';
      elsif (osch_clk = '1' and osch_clk'event) then
         timer_cnt_15ms_reg  <= timer_cnt_15ms_sig;
         timer_cnt_100us_reg <= timer_cnt_100us_sig;
      end if;
   end process TIMER_CNT_REG_PROC;

   ---------------------------------------------------
   -- DELAY_100US_REG_PROC process (sys_clk domain!!!)
   ---------------------------------------------------
   DELAY_100US_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         delay_100us_reg0 <= '0';
         delay_100us_reg1 <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         delay_100us_reg0 <= delay_100_us;
         delay_100us_reg1 <= delay_100us_reg0;
      end if;
   end process DELAY_100US_REG_PROC;

   -- DELAY_100US output
   delay_100us <= delay_100us_reg1;

   ---------------------
   -- TFT ON/OFF section
   ---------------------

   --------------------------------------------------
   -- TFT_ON_OFF_REG_PROC process (tft_clk domain!!!)
   --------------------------------------------------
   TFT_ON_OFF_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         tft_on_reg1  <= '0';
         tft_on_reg2  <= '0';
         tft_off_reg1 <= '0';
         tft_off_reg2 <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
         tft_on_reg1  <= tft_on;
         tft_on_reg2  <= tft_on_reg1;
         tft_off_reg1 <= tft_off;
         tft_off_reg2 <= tft_off_reg1;
      end if;
   end process TFT_ON_OFF_REG_PROC;

   frame_cnt_rst <= (sys_rst or tft_on_reg2 or tft_off_reg2);

   --------------------------------------------------------------
   -- FRAME_CNT_PROC process (free running and tft_clk domain!!!)
   --------------------------------------------------------------
   FRAME_CNT_PROC: process(frame_cnt_rst, tft_clk)
   begin
      if (frame_cnt_rst = '1') then
         frame_cnt <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			if (tft_vtick = '1') then
				frame_cnt <= frame_cnt + 1;
			end if;
      end if;
   end process FRAME_CNT_PROC;
	
	----------------------------------------------
	-- FRAME_CNT thresholds depend on TFT supplier
	----------------------------------------------

	------------------------------------------------------------
	-- HITACHI TX26D12
	--
	-- frame period = 1024 x 628 x 25 ns = 12.000.000 ns = 12 ms
	--
	-- frame_cnt_3  = 36 ms
	-- frame cnt_7  = 84 ms
	-- frame_cnt_15 = 180 ms
	------------------------------------------------------------

   frame_cnt_3_sig <= '1' when (frame_cnt = "0011") else
                      '0';

   frame_cnt_7_sig <= '1' when (frame_cnt = "0111") else
                      '0';

   frame_cnt_15_sig <= '1' when (frame_cnt = "1111") else
                       '0';

   -----------------------------
   -- FRAME_CNT_REG_PROC process
   -----------------------------
   FRAME_CNT_REG_PROC: process(frame_cnt_rst, tft_clk)
   begin
      if (frame_cnt_rst = '1') then
         frame_cnt_3_reg  <= '0';
         frame_cnt_7_reg  <= '0';
         frame_cnt_15_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
         frame_cnt_3_reg  <= frame_cnt_3_sig;
         frame_cnt_7_reg  <= frame_cnt_7_sig;
         frame_cnt_15_reg <= frame_cnt_15_sig;
      end if;
   end process FRAME_CNT_REG_PROC;

   ------------------------------
   -- TFT_ON_OFF_FSM_PROC process
   ------------------------------
   TFT_ON_OFF_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         tft_on_off_st <= IDLE;
         tft_vdd_en    <= '0';
         tft_avdd_en   <= '0';
         tft_disp_en   <= '0';
         tft_sig_en    <= '0';
         tft_rgb_en    <= '0';
         tft_led_en    <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
         case tft_on_off_st is
            when IDLE =>
               -- ON sequence begins
               -- ON/enable VDD digital power
               if (tft_off_reg2 = '1') then
                  tft_on_off_st <= ON_VDD;
                  tft_vdd_en    <= '1';
               end if;
               tft_avdd_en <= '0';
               tft_disp_en <= '0';
               tft_sig_en  <= '0';
               tft_rgb_en  <= '0';
               tft_led_en  <= '0';
            when ON_VDD =>
               -- ON/enable AVDD analog power
               tft_on_off_st <= ON_AVDD;
               tft_avdd_en   <= '1';
            when ON_AVDD =>
               -- ON/enable signals and RGB data
					-- HITACHI TX26D12 t1 + t2 <= 60 ms
               if (frame_cnt_3_reg = '1') then
                  tft_on_off_st <= ON_SIG_RGB;
                  tft_sig_en    <= '1';
                  tft_rgb_en    <= '1';
               end if;
            when ON_SIG_RGB =>
               -- ON/enable display e LED backlight
					-- HITACHI TX26D12 t3 > 100 ms
               if (frame_cnt_15_reg = '1') then
                  tft_on_off_st <= ON_DISP;
                  tft_disp_en   <= '1';
                  tft_led_en    <= '1';
               end if;
            when ON_DISP =>
               -- OFF sequence begins
               -- OFF display e LED backlight
               if (tft_off_reg2 = '1') then
                  tft_on_off_st <= OFF_DISP;
                  tft_disp_en   <= '0';
                  tft_led_en    <= '0';
               end if;
            when OFF_DISP =>
               -- OFF/disable signals and RGB data (t3 >= 10 frame)
					-- HITACHI TX26D12 t4 > 5 ms
               if (frame_cnt_3_reg = '1') then
                  tft_on_off_st <= OFF_SIG_RGB;
                  tft_sig_en    <= '0';
                  tft_rgb_en    <= '0';
               end if;
            when OFF_SIG_RGB =>
               -- OFF/disable AVDD analog power
               tft_on_off_st <= OFF_AVDD;
               tft_avdd_en   <= '0';
            when OFF_AVDD =>
               -- OFF/disable VDD digital power
					-- HITACHI TX26D12 t5 + t6 >= 65 ns
               if (frame_cnt_7_reg = '1') then
                  tft_on_off_st <= OFF_VDD;
                  tft_vdd_en    <= '0';
               end if;
            when OFF_VDD =>
               tft_on_off_st <= IDLE;
               tft_vdd_en    <= '0';
               tft_avdd_en   <= '0';
               tft_disp_en   <= '0';
               tft_sig_en    <= '0';
               tft_rgb_en    <= '0';
               tft_led_en    <= '0';
            when others =>
               tft_on_off_st <= IDLE;
               tft_vdd_en    <= '0';
               tft_avdd_en   <= '0';
               tft_disp_en   <= '0';
               tft_sig_en    <= '0';
               tft_rgb_en    <= '0';
               tft_led_en    <= '0';
         end case;
      end if;
   end process TFT_ON_OFF_FSM_PROC;

end rtl_power_manager;
-----------------------------------------------------------------------