-- ====================================================================
-- SDRAM_CTRL.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Company: MB
-- ====================================================================
-- Revision:   ver 0.1 - date 16 giugno 2007
--             ver 1.0 - date 04 luglio 2007
-- ====================================================================
-- Note:
--
-- BURST: 8 write and 8 read with autoprecharge
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.sdram_pkg.all;
-----------------------------------------------------------------------
entity sdram_ctrl is
   port(
      -- system interface
      sys_clk         : in std_logic;
      sys_rst         : in std_logic;
      sys_sdram_init  : in std_logic;
      sys_sdram_ready : out std_logic;
      sys_r_wn        : in std_logic;
      sys_adsn        : in std_logic;
      -- MICRON 64Mbit: system address definition
      -- bank(21-20) -> 4
      -- row(19-8)   -> 4096
      -- column(7-0) -> 256
      sys_addr        : in std_logic_vector(21 downto 0);
      sys_be          : in std_logic_vector(1 downto 0); -- active low (when high data is masked
      -- write data
      sys_data_in     : in std_logic_vector(15 downto 0);
      -- read data and data valid
      sys_data_out    : out std_logic_vector(15 downto 0);
      sys_data_val    : out std_logic;
      -- read or write cycle is completed
      sys_cycle_end   : out std_logic;
      -- auto refresh
      sys_ref_req     : in std_logic;
      sys_ref_ack     : out std_logic;
      -- sdram interface
      sdram_dq        : inout std_logic_vector(15 downto 0);
      sdram_a         : out std_logic_vector(11 downto 0);
      sdram_ba        : out std_logic_vector(1 downto 0);
      sdram_cke       : out std_logic;
      sdram_csn       : out std_logic;
      sdram_rasn      : out std_logic;
      sdram_casn      : out std_logic;
      sdram_wen       : out std_logic;
      sdram_dqm       : out std_logic_vector(1 downto 0)
   );
end sdram_ctrl;
-----------------------------------------------------------------------
architecture rtl_sdram_ctrl of sdram_ctrl is

   type sdram_fsm_state is (INIT_IDLE_NOP, INIT_PRE_ALL, INIT_TRP1, INIT_TRP2,
                            INIT_AUTO_REF1, INIT_AUTO_REF2,
                            INIT_TRFC1, INIT_TRFC2, INIT_TRFC3, INIT_TRFC4,
									 INIT_TRFC5, INIT_TRFC6, INIT_TRFC7, INIT_TRFC8,
                            INIT_LMR, INIT_TMRD,
                            CMD_IDLE_NOP, CMD_AUTO_REF,
                            CMD_TRFC1, CMD_TRFC2, CMD_TRFC3, CMD_TRFC4,
									 CMD_TRFC5, CMD_TRFC6, CMD_TRFC7, CMD_TRFC8,
                            CMD_ACTIVE, CMD_TRCD,
                            CMD_READ_AUTO, CMD_CAS1, CMD_CAS2, CMD_CAS3,
                            CMD_READ1, CMD_READ2, CMD_READ3,
                            CMD_READ4, CMD_READ5, CMD_READ6, CMD_READ7,
                            CMD_WRITE_AUTO, CMD_WRITE1, CMD_WRITE2, CMD_WRITE3,
                            CMD_WRITE4, CMD_WRITE5, CMD_WRITE6, CMD_WRITE7,
                            CMD_TWR_TRP1, CMD_TWR_TRP2);

   signal sdram_fsm_st : sdram_fsm_state;

   signal sdram_ready_reg    : std_logic;
   signal cycle_end_reg      : std_logic;
   signal ref_ack_reg        : std_logic;
   signal init_auto_ref_flag : std_logic;

   signal sdram_cmd_reg : std_logic_vector(3 downto 0);
   signal sdram_cke_reg : std_logic;
   signal sdram_ba_reg  : std_logic_vector(1 downto 0);
   signal sdram_a_reg   : std_logic_vector(11 downto 0);
   signal sdram_dqm_reg : std_logic_vector(1 downto 0);
   
   signal sys_r_wn_reg0 : std_logic;
   signal sys_r_wn_reg1 : std_logic;
   signal sys_r_wn_reg2 : std_logic;
   
   signal sys_addr_reg0 : std_logic_vector(21 downto 0);
   signal sys_addr_reg1 : std_logic_vector(21 downto 0);
   signal sys_addr_reg2 : std_logic_vector(21 downto 0);
   
   signal sys_be_reg0 : std_logic_vector(1 downto 0);
   signal sys_be_reg1 : std_logic_vector(1 downto 0);
   signal sys_be_reg2 : std_logic_vector(1 downto 0);
   signal sys_data_reg0 : std_logic_vector(15 downto 0);
   signal sys_data_reg1 : std_logic_vector(15 downto 0);
   signal sys_data_reg2 : std_logic_vector(15 downto 0);

   signal sys_data_to_sdram : std_logic;
   signal sdram_en_dq_reg   : std_logic;
   signal sdram_dq_reg      : std_logic_vector(15 downto 0);
   signal sys_data_en_reg   : std_logic;
   signal sdram_to_sys_data : std_logic;

   signal sys_sdram_init_reg : std_logic;
   signal sys_ref_req_reg    : std_logic;
   signal sys_adsn_reg       : std_logic;

	attribute syn_preserve : boolean;
	attribute syn_preserve of sdram_dq_reg  : signal is true;
	attribute syn_preserve of sys_data_reg0 : signal is true;
	attribute syn_preserve of sys_data_reg1 : signal is true;
	attribute syn_preserve of sys_data_reg2 : signal is true;

begin

   ------------------------------------------------------------------
   -- registrazione dell'interfaccia SYS:
   -- 1 solo colpo di SYS_CLK nell'ipotesi di stesso dominio di clock
   ------------------------------------------------------------------
   SYS_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_sdram_init_reg <= '0';
         sys_ref_req_reg    <= '0';
         sys_adsn_reg       <= '1';
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_sdram_init_reg <= sys_sdram_init;
         sys_ref_req_reg    <= sys_ref_req;
         sys_adsn_reg       <= sys_adsn;
      end if;
   end process SYS_REG_PROC;

   ---------------------------------------------------------------------
   -- SYS_ADSN e SYS_R_WN sono sincroni all'interfaccia SYS:
   -- devo quindi ritardare sino allo stato CMD_TRCD il segnale SYS_R_WN
   -- per valutare la richiesta di lettura o scrittura
   -- Stesso discorso per SYS_ADDR, SYS_BE e SYS_DATA_IN
   ---------------------------------------------------------------------
   SYS_DELAY_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_r_wn_reg0 <= '1';
         sys_r_wn_reg1 <= '1';
         sys_r_wn_reg2 <= '1';
         sys_addr_reg0 <= (others => '1');
         sys_addr_reg1 <= (others => '1');
         sys_addr_reg2 <= (others => '1');
         sys_be_reg0   <= (others => '0');
         sys_be_reg1   <= (others => '0');
         sys_be_reg2   <= (others => '0');
         sys_data_reg0 <= (others => '0');
         sys_data_reg1 <= (others => '0');
         sys_data_reg2 <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_r_wn_reg0 <= sys_r_wn;
         sys_r_wn_reg1 <= sys_r_wn_reg0;
         sys_r_wn_reg2 <= sys_r_wn_reg1;
         sys_addr_reg0 <= sys_addr;
         sys_addr_reg1 <= sys_addr_reg0;
         sys_addr_reg2 <= sys_addr_reg1;
         sys_be_reg0   <= sys_be;
         sys_be_reg1   <= sys_be_reg0;
         sys_be_reg2   <= sys_be_reg1;
         sys_data_reg0 <= sys_data_in;
         sys_data_reg1 <= sys_data_reg0;
         sys_data_reg2 <= sys_data_reg1;
      end if;
   end process SYS_DELAY_REG_PROC;

   ------------
   -- SDRAM FSM
   ------------
   SDRAM_FSM_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sdram_fsm_st       <= INIT_IDLE_NOP;
         sdram_ready_reg    <= '0';
         cycle_end_reg      <= '0';
         ref_ack_reg        <= '0';
         init_auto_ref_flag <= '0';
         sys_data_to_sdram  <= '0';
         sdram_to_sys_data  <= '0';
         -- SDRAM command
         sdram_cmd_reg <= C_INHIBIT;
         sdram_cke_reg <= '0';
         sdram_ba_reg  <= (others => '1');
         sdram_a_reg   <= (others => '1');
         sdram_dqm_reg <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
         -- default
         cycle_end_reg     <= '0';
         ref_ack_reg       <= '0';
         sys_data_to_sdram <= '0';
         sdram_to_sys_data <= '0';
         -- SDRAM NOP command
         sdram_cmd_reg <= C_NOP;
         sdram_cke_reg <= '1';
         sdram_ba_reg  <= (others => '1');
         sdram_a_reg   <= (others => '1');
         sdram_dqm_reg <= (others => '0');
         -- fsm
         case sdram_fsm_st is
            -- INIT_IDLE_NOP
            when INIT_IDLE_NOP =>
               if (sys_sdram_init_reg = '1') then
                  sdram_fsm_st  <= INIT_PRE_ALL;
                  sdram_cmd_reg <= C_PRECHARGE;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= (others => '1');
                  sdram_a_reg   <= (others => '1');
               else
                  sdram_fsm_st <= INIT_IDLE_NOP;
               end if;
            -- INIT_PRE_ALL
            when INIT_PRE_ALL =>
               sdram_fsm_st <= INIT_TRP1;
            -- INIT_TRP1
            when INIT_TRP1 =>
               sdram_fsm_st <= INIT_TRP2;
            -- INIT_TRP2
            when INIT_TRP2 =>
               sdram_fsm_st  <= INIT_AUTO_REF1;
               sdram_cmd_reg <= C_AUTO_REFRESH;
               sdram_cke_reg <= '1';
               sdram_ba_reg  <= (others => '1');
               sdram_a_reg   <= (others => '1');
            -- INIT_AUTO_REF1
            when INIT_AUTO_REF1 =>
               sdram_fsm_st       <= INIT_TRFC1;
               init_auto_ref_flag <= '0';
            -- INIT_TRFC1
            when INIT_TRFC1 =>
               sdram_fsm_st    <= INIT_TRFC2;
            -- INIT_TRFC2
            when INIT_TRFC2 =>
               sdram_fsm_st    <= INIT_TRFC3;
            -- INIT_TRFC3
            when INIT_TRFC3 =>
               sdram_fsm_st    <= INIT_TRFC4;
            -- INIT_TRFC4
            when INIT_TRFC4 =>
               sdram_fsm_st    <= INIT_TRFC5;
            -- INIT_TRFC5
            when INIT_TRFC5 =>
               sdram_fsm_st    <= INIT_TRFC6;
            -- INIT_TRFC6
            when INIT_TRFC6 =>
               sdram_fsm_st    <= INIT_TRFC7;
            -- INIT_TRFC7
            when INIT_TRFC7 =>
               sdram_fsm_st    <= INIT_TRFC8;
				-- INIT_TRFC8
            when INIT_TRFC8 =>
               if (init_auto_ref_flag = '0') then
                  sdram_fsm_st  <= INIT_AUTO_REF2;
                  sdram_cmd_reg <= C_AUTO_REFRESH;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= (others => '1');
                  sdram_a_reg   <= (others => '1');
               else
                  sdram_fsm_st <= INIT_LMR;
                  sdram_cmd_reg <= C_LOAD_MODE_REG;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= (others => '0');
                  sdram_a_reg   <= ("00" &
                                    MR_WRITE_BURST &
                                    MR_OPERATION &
                                    MR_CAS_LATENCY &
                                    MR_BURST_TYPE &
                                    MR_BURST_LENGTH);
               end if;
            -- INIT_AUTO_REF2
            when INIT_AUTO_REF2 =>
               sdram_fsm_st       <= INIT_TRFC1;
               init_auto_ref_flag <= '1';
            -- INIT_LMR
            when INIT_LMR =>
               sdram_fsm_st <= INIT_TMRD;
            -- INIT_TMRD
            when INIT_TMRD =>
               sdram_fsm_st    <= CMD_IDLE_NOP;
               sdram_ready_reg <= '1'; 
            -- CMD_IDLE_NOP
            when CMD_IDLE_NOP =>
               if (sys_ref_req_reg = '1') then
                  sdram_fsm_st  <= CMD_AUTO_REF;
                  sdram_cmd_reg <= C_AUTO_REFRESH;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= (others => '1');
                  sdram_a_reg   <= (others => '1');
               elsif (sys_adsn_reg = '0') then
                  sdram_fsm_st  <= CMD_ACTIVE;
                  sdram_cmd_reg <= C_ACTIVE;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= sys_addr_reg0(21 downto 20); -- bank
                  sdram_a_reg   <= sys_addr_reg0(19 downto 8);  -- row
               end if;
            -- CMD_AUTO_REF
            when CMD_AUTO_REF =>
               sdram_fsm_st <= CMD_TRFC1;
            -- CMD_TRFC1
            when CMD_TRFC1 =>
               sdram_fsm_st <= CMD_TRFC2;
            -- CMD_TRFC2
            when CMD_TRFC2 =>
               sdram_fsm_st <= CMD_TRFC3;
            -- CMD_TRFC3
            when CMD_TRFC3 =>
               sdram_fsm_st <= CMD_TRFC4;
            -- CMD_TRFC4
            when CMD_TRFC4 =>
               sdram_fsm_st <= CMD_TRFC5;
            -- CMD_TRFC5
            when CMD_TRFC5 =>
               sdram_fsm_st <= CMD_TRFC6;
            -- CMD_TRFC6
            when CMD_TRFC6 =>
               sdram_fsm_st <= CMD_TRFC7;
            -- CMD_TRFC7
            when CMD_TRFC7 =>
               sdram_fsm_st <= CMD_TRFC8;
            -- CMD_TRFC8
            when CMD_TRFC8 =>
               ref_ack_reg  <= '1';
               -- !!!
               -- se avessi mantenuto la transizione di stato
               -- sdram_fsm_st <= CMD_IDLE_NOP;
               -- il READ avrebbe richiesto 13 clk
               -- Prima di poter effettuare una nuova operazione (ACTIVE)
               -- attendevo infatti un colpo di clock dato dallo stato CMD_IDLE_NOP
               -- Modifico per poter ripartire subito con la valutazione
               -- di un nuovo comando di AUTOREFRESH, READ o WRITE con AUTOPRECHARGE
               -- !!!
               if (sys_ref_req_reg = '1') then
                  sdram_fsm_st  <= CMD_AUTO_REF;
                  sdram_cmd_reg <= C_AUTO_REFRESH;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= (others => '1');
                  sdram_a_reg   <= (others => '1');
               elsif (sys_adsn_reg = '0') then
                  sdram_fsm_st  <= CMD_ACTIVE;
                  sdram_cmd_reg <= C_ACTIVE;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= sys_addr_reg0(21 downto 20); -- bank
                  sdram_a_reg   <= sys_addr_reg0(19 downto 8);  -- row
               else
                  sdram_fsm_st <= CMD_IDLE_NOP;
               end if;
            -- CMD_ACTIVE
            when CMD_ACTIVE =>
               sdram_fsm_st <= CMD_TRCD;
               if (sys_r_wn_reg1 = '0') then
                  sys_data_to_sdram <= '1';
               end if;
            -- CMD_TRCD
            when CMD_TRCD =>
               if (sys_r_wn_reg2 = '1') then
                  sdram_fsm_st <= CMD_READ_AUTO;
                  sdram_cmd_reg <= C_READ;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= sys_addr_reg2(21 downto 20); -- bank
                  sdram_a_reg   <= ('0' &                       -- sdram_a(11) = don't care
                                    '1' &                       -- sdram_a(10) = 1 enable auto precharge
                                    "00" &                      -- sdram_a(9:8) = don't care
                                    sys_addr_reg2(7 downto 0)); -- sdram_a(7:0) col
                  sdram_dqm_reg <= (others => '0');
               else
                  sdram_fsm_st <= CMD_WRITE_AUTO;
                  sys_data_to_sdram <= '1';
                  sdram_cmd_reg <= C_WRITE;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= sys_addr_reg2(21 downto 20); -- bank
                  sdram_a_reg   <= ('0' &                       -- sdram_a(11) = don't care
                                    '1' &                       -- sdram_a(10) = 1 enable auto precharge
                                    "00" &                      -- sdram_a(9:8) = don't care
                                    sys_addr_reg2(7 downto 0)); -- sdram_a(7:0) col
                  sdram_dqm_reg <= sys_be_reg2;
               end if;
            -- CMD_READ_AUTO
            when CMD_READ_AUTO =>
               sdram_fsm_st      <= CMD_CAS1;
               sdram_dqm_reg     <= (others => '0');
            -- CMD_CAS1
            when CMD_CAS1 =>
               sdram_fsm_st      <= CMD_CAS2;
               sdram_dqm_reg     <= (others => '0');
               sdram_to_sys_data <= '1';
            -- CMD_CAS2
            when CMD_CAS2 =>
               sdram_fsm_st      <= CMD_CAS3;
               sdram_dqm_reg     <= (others => '0');
               sdram_to_sys_data <= '1';
				-- CMD_CAS3
            when CMD_CAS3 =>
               sdram_fsm_st <= CMD_READ1;
               sdram_to_sys_data <= '1';
            -- CMD_READ1
            when CMD_READ1 =>
               sdram_fsm_st <= CMD_READ2;
               sdram_to_sys_data <= '1';
            -- CMD_READ2
            when CMD_READ2 =>
               sdram_fsm_st <= CMD_READ3;
               sdram_to_sys_data <= '1';
            -- CMD_READ3
            when CMD_READ3 =>
               sdram_fsm_st <= CMD_READ4;
               sdram_to_sys_data <= '1';
            -- CMD_READ4
            when CMD_READ4 =>
               sdram_fsm_st <= CMD_READ5;
               sdram_to_sys_data <= '1';
            -- CMD_READ5
            when CMD_READ5 =>
               sdram_fsm_st <= CMD_READ6;
               sdram_to_sys_data <= '1';
               cycle_end_reg <= '1';
            -- CMD_READ6
            when CMD_READ6 =>
               sdram_fsm_st  <= CMD_READ7;
               -- cycle_end_reg <= '1';
            -- CMD_READ7
            when CMD_READ7 =>
               -- !!!
               -- se avessi mantenuto la transizione di stato
               -- sdram_fsm_st <= CMD_IDLE_NOP;
               -- il READ avrebbe richiesto 13 clk
               -- Prima di poter effettuare una nuova operazione (ACTIVE)
               -- attendevo infatti un colpo di clock dato dallo stato CMD_IDLE_NOP
               -- Modifico per poter ripartire subito con la valutazione
               -- di un nuovo comando di AUTOREFRESH, READ o WRITE con AUTOPRECHARGE
               -- !!!
               if (sys_ref_req_reg = '1') then
                  sdram_fsm_st  <= CMD_AUTO_REF;
                  sdram_cmd_reg <= C_AUTO_REFRESH;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= (others => '1');
                  sdram_a_reg   <= (others => '1');
               elsif (sys_adsn_reg = '0') then
                  sdram_fsm_st  <= CMD_ACTIVE;
                  sdram_cmd_reg <= C_ACTIVE;
                  sdram_cke_reg <= '1';
                  sdram_ba_reg  <= sys_addr_reg0(21 downto 20); -- bank
                  sdram_a_reg   <= sys_addr_reg0(19 downto 8);  -- row
               else
                  sdram_fsm_st <= CMD_IDLE_NOP;
               end if;
            -- CMD_WRITE_AUTO
            when CMD_WRITE_AUTO =>
               sdram_fsm_st <= CMD_WRITE1;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE1
            when CMD_WRITE1 =>
               sdram_fsm_st <= CMD_WRITE2;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE2
            when CMD_WRITE2 =>
               sdram_fsm_st <= CMD_WRITE3;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE3
            when CMD_WRITE3 =>
               sdram_fsm_st <= CMD_WRITE4;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE4
            when CMD_WRITE4 =>
               sdram_fsm_st <= CMD_WRITE5;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE5
            when CMD_WRITE5 =>
               sdram_fsm_st <= CMD_WRITE6;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE6
            when CMD_WRITE6 =>
               sdram_fsm_st <= CMD_WRITE7;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_WRITE7
            when CMD_WRITE7 =>
               sdram_fsm_st <= CMD_TWR_TRP1;
               sdram_dqm_reg <= sys_be_reg2;
               sys_data_to_sdram <= '1';
            -- CMD_TWR_TRP1
            when CMD_TWR_TRP1 =>
               sdram_fsm_st <= CMD_TWR_TRP2;
               cycle_end_reg <= '1';
            -- CMD_TWR_TRP2
            when CMD_TWR_TRP2 =>
               sdram_fsm_st  <= CMD_IDLE_NOP;
               -- cycle_end_reg <= '1';
            -- others
            when others =>
               sdram_fsm_st  <= CMD_IDLE_NOP;
               sdram_cmd_reg <= C_NOP;
               sdram_cke_reg <= '1';
               sdram_ba_reg  <= (others => '1');
               sdram_a_reg   <= (others => '1');
         end case;
      end if;
   end process SDRAM_FSM_PROC;

   -- sys output signal

   --sys_sdram_ready <= sdram_ready_reg;
	-- 17 Sept 2011
   SDRAM_READY_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_sdram_ready <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_sdram_ready <= sdram_ready_reg;
      end if;
   end process SDRAM_READY_PROC;
	
	
   sys_cycle_end   <= cycle_end_reg;
   sys_ref_ack     <= ref_ack_reg;

   -- sdram output signals
   sdram_csn  <= sdram_cmd_reg(3) after TDLY;
   sdram_rasn <= sdram_cmd_reg(2) after TDLY;
   sdram_casn <= sdram_cmd_reg(1) after TDLY;
   sdram_wen  <= sdram_cmd_reg(0) after TDLY;
   sdram_cke  <= sdram_cke_reg after TDLY;
   sdram_ba   <= sdram_ba_reg after TDLY;
   sdram_a    <= sdram_a_reg after TDLY;
   --sdram_dqm  <= (others => '0');
   sdram_dqm  <= sdram_dqm_reg after TDLY;
   -- generazione enable registrato per pilotare IO buffer
   -- e registrazione del dato SYS -> SDRAM
   SDRAM_DATA_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sdram_en_dq_reg  <= '0';
         sdram_dq_reg     <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
         sdram_dq_reg <= sys_data_reg2;
         if (sys_data_to_sdram = '1') then
            sdram_en_dq_reg  <= '1';
         else
            sdram_en_dq_reg  <= '0';
         end if;
      end if;
   end process SDRAM_DATA_REG_PROC;

   sdram_dq <= sdram_dq_reg after TDLY when (sdram_en_dq_reg = '1') else
               (others => 'Z') after TDLY;

   SYS_DATA_EN_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_data_en_reg <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_data_en_reg <= sdram_to_sys_data;
      end if;
   end process SYS_DATA_EN_REG_PROC;

   SYS_DATA_VAL_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_data_val <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_data_val <= sys_data_en_reg;
      end if;
   end process SYS_DATA_VAL_REG_PROC;

   SYS_DATA_OUT_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         sys_data_out <= (others => '1');
      elsif (sys_clk = '1' and sys_clk'event) then
         sys_data_out <= sdram_dq;
      end if;
   end process SYS_DATA_OUT_REG_PROC;

end rtl_sdram_ctrl;
-----------------------------------------------------------------------