-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
-- Copyright (c) 2011 by Lattice Semiconductor Corporation
-- Permission:
--   Lattice Semiconductor grants permission to use this code for use
--   in synthesis for any Lattice programmable logic product.  Other
--   use of this code, including the selling or duplication of any
--   portion is strictly prohibited.
-- Disclaimer:
--   This VHDL or Verilog source code is intended as a design reference
--   which illustrates how these types of functions can be implemented.
--   It is the user's responsibility to verify their design for
--   consistency and functionality through the use of formal
--   verification methods.  Lattice Semiconductor provides no warranty
--   regarding the use or functionality of this code.                
-- ====================================================================
--						Lattice Semiconductor Corporation                
--						5555 NE Moore Court                
--						Hillsboro, OR 97214                
--						U.S.A                         
--						TEL: 1-800-Lattice (USA and Canada)                
--						503-268-8001 (other locations)                
--						web: http:--www.latticesemi.com/                
-- ====================================================================
-- >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
-- ====================================================================
--
-- ====================================================================
-- TFT_TIMING_CTRL.vhd
-- ====================================================================
-- Designer: MB
-- ====================================================================
-- Revision: ver 1.0 - date 11 July 2011
-- ====================================================================
-- Note:
-- 
-- VERTICAL SYNC
--
--    ...--- Blanking LINES -->|<-- Active LINES -->|<--...
--                             |                    |            
--                             |                    |            
-- ...--+          +-------------------- ... -------------------+
--      |   VSYNC  |    VBP    |      V Pixels      |    VFP    |
--      |<--V Sync |<--V Back  |<---- TFT Rows ---->|<--V Front |
--      |   Time-->|   Porch-->|                        Porch-->|
--      +----------+                                            +--...
--      |                       V PERIOD                        |
--      |<---------------------- FRAME ------------------------>|
-- 
-- 
-- HORIZONTAL SYNC
--
--    ...-- Blanking PIXELS -->|<-- Active PIXELS ->|<--...
--                             |                    |            
--                             |                    |            
-- ...--+          +-------------------- ... -------------------+
--      |   HSYNC  |    HBP    |      H Pixels      |    HFP    |
--      |<--H Sync |<--H Back  |<--- TFT Columns -->|<--H Front |
--      |   Time-->|   Porch-->|                        Porch-->|
--      +----------+                                            +--...
--      |                       H PERIOD                        |
--      |<--------------------- TFT ROW ----------------------->|
--
-- ====================================================================

-----------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-----------------------------------------------------------------------
library work;
use work.tft_pkg.all;
-----------------------------------------------------------------------
entity tft_timing_ctrl is
   port(
      -- system interface
      sys_rst         : in std_logic;
      sys_clk         : in std_logic;
      tft_clk         : in std_logic;
      -- microcontroller interface
      sdram_test      : in std_logic;
      tft_test        : in std_logic;
      tft_bank        : in std_logic_vector(1 downto 0);
      tft_sect        : in std_logic_vector(2 downto 0);
      -- power manager interface
      tft_sig_en      : in std_logic;
      tft_rgb_en      : in std_logic;
		tft_vtick       : out std_logic;
      -- monster manager interface
		tft_row_start   : out std_logic;
		tft_sdram_dtmg  : out std_logic;
      tft_sdram_addr  : out std_logic_vector(21 downto 0);
      tft_sdram_red   : in std_logic_vector(4 downto 0);
      tft_sdram_green : in std_logic_vector(5 downto 0);
      tft_sdram_blue  : in std_logic_vector(4 downto 0);
      -- tft interface
      tft_hsync       : out std_logic;
      tft_vsync       : out std_logic;
		tft_dtmg        : out std_logic;
      tft_red         : out std_logic_vector(7 downto 0);
      tft_green       : out std_logic_vector(7 downto 0);
      tft_blue        : out std_logic_vector(7 downto 0)
   );
end tft_timing_ctrl;
-----------------------------------------------------------------------
architecture rtl_tft_timing_ctrl of tft_timing_ctrl is

   signal hsync_cnt : std_logic_vector(15 downto 0); -- 16 bit!!!
   signal vsync_cnt : std_logic_vector(15 downto 0); -- 16 bit!!!

   signal hsync_cnt_reg : std_logic_vector(15 downto 0); -- 16 bit!!!
   signal vsync_cnt_reg : std_logic_vector(15 downto 0); -- 16 bit!!!

   signal h_tick_reg : std_logic;
   signal v_tick_reg : std_logic;

   type vsync_state is (VSYNC_ST0, VSYNC_ST1);
   signal vsync_st : vsync_state;

   signal vsync_reg : std_logic;

   type vsync_to_hsync_state is (VSYNC_TO_HSYNC_ST0, VSYNC_TO_HSYNC_ST1);
   signal vsync_to_hsync_st : vsync_to_hsync_state;

	signal vsync_to_hsync_reg : std_logic;
	
   type hsync_state is (HSYNC_ST0, HSYNC_ST1);
   signal hsync_st : hsync_state;

   signal hsync_reg : std_logic;

   type hdata_state is (HDATA_ST0, HDATA_ST1);
   signal hdata_st : hdata_state;

	signal hdata_reg : std_logic;

   signal debug_hsync_st : hsync_state;

   signal debug_hsync_reg : std_logic;

   signal rst_pixel_cnt : std_logic;

   signal pixel_cnt  : std_logic_vector(18 downto 0); -- 19 bit!!!
   signal column_cnt : std_logic_vector(9 downto 0);
   signal row_cnt    : std_logic_vector(9 downto 0);

	signal tft_test_red   : std_logic_vector(7 downto 0);
	signal tft_test_green : std_logic_vector(7 downto 0);
	signal tft_test_blue  : std_logic_vector(7 downto 0);

   -- signal vsync_reg0 : std_logic;
   -- signal hsync_reg0 : std_logic;
   -- signal hdata_reg0 : std_logic;
	signal vsync_final_sr : std_logic_vector(15 downto 0);
	signal hsync_final_sr : std_logic_vector(15 downto 0);
	signal hdata_final_sr : std_logic_vector(15 downto 0);

   signal pixel_hdata_reg : std_logic;

   signal pixel_hdata_sr0 : std_logic;
   signal pixel_hdata_sr1 : std_logic;
   signal pixel_hdata_sr2 : std_logic;

   signal pixel_hdata_sr_sig : std_logic;
   signal pixel_hdata_sr_reg : std_logic;

   signal tft_pixel_reg : std_logic_vector(18 downto 0); -- 19 bit!!!

   signal v_tick_reg_sr : std_logic_vector(3 downto 0);

   signal tft_test_reg0   : std_logic;
   signal tft_test_reg1   : std_logic;
   signal sdram_test_reg0 : std_logic;
   signal sdram_test_reg1 : std_logic;

   signal tft_bank_reg : std_logic_vector(1 downto 0);
   signal tft_sect_reg : std_logic_vector(2 downto 0);

   signal hdata_sr : std_logic_vector(7 downto 0);

   signal hdata_sr_sig : std_logic;
   signal hdata_sr_reg : std_logic;

   signal frame_cnt : std_logic_vector(9 downto 0);

begin

	-------------------------------------------------------------
	-- SPOSTARE QUESTO PROCESSO NELL'INTERFACCIA MICROCONTROLLORE
	-------------------------------------------------------------
   -----------------------------
   -- TFT_UPDATE_SR_PROC process
   -----------------------------
   -- TFT_UPDATE_SR_PROC: process(sys_rst, sys_clk)
   -- begin
      -- if (sys_rst = '1') then
			-- tft_update_sr <= (others => '0');
      -- elsif (sys_clk = '1' and sys_clk'event) then
			-- tft_update_sr(0)          <= tft_update;
			-- tft_update_sr(7 downto 1) <= tft_update_sr(6 downto 0);
      -- end if;
   -- end process TFT_UPDATE_SR_PROC;
   -----------------------------
   -- TFT_UPDATE_SR_PROC process
   -----------------------------
	-------------------------------------------------------------
	-- SPOSTARE QUESTO PROCESSO NELL'INTERFACCIA MICROCONTROLLORE
	-------------------------------------------------------------

   ----------------------
   -- HV_SYNC_CNT process
   ----------------------
   HV_SYNC_CNT_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         hsync_cnt  <= (others => '0');
			vsync_cnt  <= (others => '0');
			h_tick_reg <= '0';
			v_tick_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			v_tick_reg <= '0';
			h_tick_reg <= '0';
         if (hsync_cnt = (H_PERIOD - 1)) then
            hsync_cnt  <= (others => '0');
				h_tick_reg <= '1';
				if (vsync_cnt = (V_PERIOD - 1)) then
					vsync_cnt  <= (others => '0');
					v_tick_reg <= '1';
				else
					vsync_cnt <= vsync_cnt + 1;
				end if;
         else
				hsync_cnt <= hsync_cnt + 1;
         end if;
      end if;
   end process HV_SYNC_CNT_PROC;

	-- TFT_VTICK output to power_manager component
	tft_vtick <= v_tick_reg;

   --------------------------
   -- HV_SYNC_CNT_REG process
   --------------------------
   HV_SYNC_CNT_REG_PROC: process(tft_clk)
   begin
      if (tft_clk = '1' and tft_clk'event) then
			hsync_cnt_reg <= hsync_cnt;
			vsync_cnt_reg <= vsync_cnt;
      end if;
   end process HV_SYNC_CNT_REG_PROC;

   --------------------
   -- VSYNC_FSM process
   --------------------
   VSYNC_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         vsync_st  <= VSYNC_ST0;
			vsync_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			if (h_tick_reg = '1') then
				case vsync_st is
					when VSYNC_ST0 =>
						if (vsync_cnt_reg = (V_SYNCTIME - 1)) then
							vsync_st  <= VSYNC_ST1;
							vsync_reg <= '1';
						end if;
					when VSYNC_ST1 =>
						if (vsync_cnt_reg = (V_PERIOD - 1)) then
							vsync_st  <= VSYNC_ST0;
							vsync_reg <= '0';
						end if;
					when others =>
						vsync_st  <= VSYNC_ST0;
						vsync_reg <= '0';
				end case;
			end if;
		end if;
   end process VSYNC_FSM_PROC;

   -----------------------------
   -- VSYNC_TO_HSYNC_FSM process
   -----------------------------
   VSYNC_TO_HSYNC_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         vsync_to_hsync_st  <= VSYNC_TO_HSYNC_ST0;
			vsync_to_hsync_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			if (h_tick_reg = '1') then
				case vsync_to_hsync_st is
					when VSYNC_TO_HSYNC_ST0 =>
						if (vsync_cnt_reg = (V_SYNCTIME + V_BACKPORCH - 1)) then
							vsync_to_hsync_st  <= VSYNC_TO_HSYNC_ST1;
							vsync_to_hsync_reg <= '1';
						end if;
					when VSYNC_TO_HSYNC_ST1 =>
						if (vsync_cnt_reg = (V_SYNCTIME + V_BACKPORCH + V_LINES - 1)) then
							vsync_to_hsync_st  <= VSYNC_TO_HSYNC_ST0;
							vsync_to_hsync_reg <= '0';
						end if;
					when others =>
						vsync_to_hsync_st  <= VSYNC_TO_HSYNC_ST0;
						vsync_to_hsync_reg <= '0';
				end case;
			end if;
		end if;
   end process VSYNC_TO_HSYNC_FSM_PROC;

   --------------------
   -- HSYNC_FSM process
   --------------------
   HSYNC_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			hsync_st  <= HSYNC_ST0;
         hsync_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			if (vsync_to_hsync_reg = '1') then
				case hsync_st is
					when HSYNC_ST0 =>
						if (hsync_cnt_reg = (H_SYNCTIME - 1)) then
							hsync_st  <= HSYNC_ST1;
							hsync_reg <= '1';
						end if;
					when HSYNC_ST1 =>
						if (hsync_cnt_reg = (H_PERIOD - 1)) then
							hsync_st  <= HSYNC_ST0;
							hsync_reg <= '0';
						end if;
					when others =>
						hsync_st  <= HSYNC_ST0;
						hsync_reg <= '0';
				end case;
			else
				hsync_st  <= HSYNC_ST0;
				hsync_reg <= '0';
			end if;
		end if;
   end process HSYNC_FSM_PROC;

   --------------------------
   -- DEBUG HSYNC_FSM process 
   --------------------------
   DEBUG_HSYNC_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			debug_hsync_st  <= HSYNC_ST0;
         debug_hsync_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			case debug_hsync_st is
				when HSYNC_ST0 =>
					if (hsync_cnt_reg = (H_SYNCTIME - 1)) then
						debug_hsync_st  <= HSYNC_ST1;
						debug_hsync_reg <= '1';
					end if;
				when HSYNC_ST1 =>
					if (hsync_cnt_reg = (H_PERIOD - 1)) then
						debug_hsync_st  <= HSYNC_ST0;
						debug_hsync_reg <= '0';
					end if;
				when others =>
					debug_hsync_st  <= HSYNC_ST0;
					debug_hsync_reg <= '0';
			end case;
		end if;
   end process DEBUG_HSYNC_FSM_PROC;

   --------------------
   -- HDATA_FSM process
   --------------------
   HDATA_FSM_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			hdata_st  <= HDATA_ST0;
         hdata_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			if (vsync_to_hsync_reg = '1') then
				case hdata_st is
					when HDATA_ST0 =>
						if (hsync_cnt_reg = (H_SYNCTIME + H_BACKPORCH - 1)) then
							hdata_st  <= HDATA_ST1;
							hdata_reg <= '1';
						end if;
					when HDATA_ST1 =>
						if (hsync_cnt_reg = (H_SYNCTIME + H_PIXELS + H_BACKPORCH - 1)) then
							hdata_st  <= HDATA_ST0;
							hdata_reg <= '0';
						end if;
					when others =>
						hdata_st  <= HDATA_ST0;
						hdata_reg <= '0';
				end case;
			else
				hdata_st  <= HDATA_ST0;
				hdata_reg <= '0';
			end if;
		end if;
   end process HDATA_FSM_PROC;

   rst_pixel_cnt <= (sys_rst or v_tick_reg);

   -----------------------
   -- COUNTER_PROC process
   -----------------------
   COUNTER_PROC: process(rst_pixel_cnt, tft_clk)
   begin
      if (rst_pixel_cnt = '1') then
         pixel_cnt       <= (others => '0');
         column_cnt      <= (others => '0');
         row_cnt         <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
         if (hdata_reg = '1') then
				-- COULUMN and ROW increment
            if (column_cnt = (H_PIXELS - 1)) then
               column_cnt <= (others => '0');
               if (row_cnt = (V_LINES - 1)) then
                  row_cnt <= (others => '0');
               else
                  row_cnt <= row_cnt + 1;
               end if;
            else
               column_cnt <= column_cnt + 1;
            end if;
				-- PIXEL increment 
            if (pixel_cnt = (TOTAL_PIXEL - 1)) then
               pixel_cnt <= (others => '0');
            else
               pixel_cnt <= pixel_cnt + 1;
            end if;
         end if;
		end if;
   end process COUNTER_PROC;

   ---------------------------------------------------
   -- PIXEL_HDATA_REG_PROC process (tft_clk domain!!!)
   ---------------------------------------------------
   PIXEL_HDATA_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			pixel_hdata_reg <= '0';
      elsif (tft_clk = '1' and tft_clk'event) then
			pixel_hdata_reg <= (hdata_reg and not pixel_cnt(2));
      end if;
   end process PIXEL_HDATA_REG_PROC;

   --------------------------------------------------
   -- PIXEL_HDATA_SR_PROC process (sys_clk domain!!!)
   --------------------------------------------------
   PIXEL_HDATA_SR_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
			pixel_hdata_sr0 <= '0';
			pixel_hdata_sr1 <= '0';
			pixel_hdata_sr2 <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
			pixel_hdata_sr0 <= pixel_hdata_reg;
			pixel_hdata_sr1 <= pixel_hdata_sr0;
			pixel_hdata_sr2 <= pixel_hdata_sr1;
      end if;
   end process PIXEL_HDATA_SR_PROC;

	-- use registers after double sampling
	pixel_hdata_sr_sig <= '1' when ((pixel_hdata_sr1 = '1') and (pixel_hdata_sr2 = '0')) else
	                      '0';

   ----------------------------------
   -- PIXEL_HDATA_SR_REG_PROC process
   ----------------------------------
   PIXEL_HDATA_SR_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
			pixel_hdata_sr_reg <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
			pixel_hdata_sr_reg <= pixel_hdata_sr_sig;
      end if;
   end process PIXEL_HDATA_SR_REG_PROC;

   -----------------------------
   -- TFT_PIXEL_REG_PROC process
   -----------------------------
   TFT_PIXEL_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         tft_pixel_reg <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
			if (pixel_hdata_sr_reg = '1') then
				tft_pixel_reg <= pixel_cnt;
			end if;
      end if;
   end process TFT_PIXEL_REG_PROC;

   -------------------------------------------------
   -- V_TICK_REG_SR_PROC process (sys_clk domain!!!)
   -------------------------------------------------
   V_TICK_REG_SR_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
			v_tick_reg_sr <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
			v_tick_reg_sr(0)          <= v_tick_reg;
			v_tick_reg_sr(3 downto 1) <= v_tick_reg_sr(2 downto 0);
      end if;
   end process V_TICK_REG_SR_PROC;

   --------------------------------------------
   -- TEST_REG_PROC process (sys_clk domain!!!)
   --------------------------------------------
   TEST_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         tft_test_reg0   <= '0';
         tft_test_reg1   <= '0';
         sdram_test_reg0 <= '0';
         sdram_test_reg1 <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
			if (v_tick_reg = '1') then
				tft_test_reg0   <= tft_test;
				tft_test_reg1   <= tft_test_reg0;
				sdram_test_reg0 <= sdram_test;
				sdram_test_reg1 <= sdram_test_reg0;
			end if;
      end if;
   end process TEST_REG_PROC;

   --------------------------------------------------------------
   -- FRAME_CNT_PROC process (free running and tft_clk domain!!!)
   --------------------------------------------------------------
   FRAME_CNT_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         frame_cnt <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			if (v_tick_reg = '1') then
				frame_cnt <= frame_cnt + 1;
			end if;
      end if;
   end process FRAME_CNT_PROC;

   ---------------------------------
   -- TFT_BANK_SECT_REG_PROC process
   ---------------------------------
   TFT_BANK_SECT_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
         tft_bank_reg  <= (others => '0');
         tft_sect_reg  <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
			if (v_tick_reg_sr(3) = '1') then
            if (sdram_test_reg1 = '1') then
               tft_bank_reg <= (others => '0'); -- TBD!!!
               tft_sect_reg <= (others => '0'); -- TBD!!!
            else
                tft_bank_reg <= tft_bank;
                tft_sect_reg <= tft_sect;
               --tft_bank_reg <= (others => '0');
               --tft_sect_reg <= (others => '0');
               --tft_bank_reg <= frame_cnt(8 downto 7);
               --tft_sect_reg <= ("00" & frame_cnt(6));
			   
               --tft_bank_reg <= frame_cnt(9 downto 8);
               --tft_sect_reg <= ("00" & frame_cnt(7));
            end if;
         end if;
      end if;
   end process TFT_BANK_SECT_REG_PROC;
	
   ----------------------------------------------------------
   -- SDRAM - system address definition
   --    4 bank   -  2 bit - bank(21:20)
   -- 4096 row    - 12 bit - row(19:8)
   --  256 column -  8 bit - column(7:0)
   ----------------------------------------------------------
   -- Mapping uC address registers -> SDRAM address
   -- bank(21:20) = REG0(5:4)
   -- row(19:8)   = REG0(3) & REG0(2:1) & REG0(0) & REG1(7:0)
   -- column(7:0) = REG2(7:0)
   ----------------------------------------------------------
	-- resolution 800x600
   -- bank(21:20) = REG0(5:4) = B1 B0
   -- row(19:8)   = REG0(3)   = S0
	--               REG0(2:1) = A18 A17
	--               REG0(0)   = A16
	--               REG1(7:0) = A15 ... A8
   -- column(7:0) = REG2(7:0) = A7 ... A0
   ----------------------------------------------------------
   tft_sdram_addr <= (tft_bank_reg & tft_sect_reg(0) & tft_pixel_reg(18 downto 3) & "000");

   --------------------------------------------
   -- HDATA_SR_PROC process (sys_clk DOMAIN!!!)
   --------------------------------------------
   -- HDATA_SR_PROC: process(sys_rst, sys_clk)
   -- begin
      -- if (sys_rst = '1') then
			-- hdata_sr0 <= '0';
			-- hdata_sr1 <= '0';
			-- hdata_sr2 <= '0';
      -- elsif (sys_clk = '1' and sys_clk'event) then
			-- hdata_sr0 <= hdata_reg;
			-- hdata_sr1 <= hdata_sr0;
			-- hdata_sr2 <= hdata_sr1;
      -- end if;
   -- end process HDATA_SR_PROC;
   HDATA_SR_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
			hdata_sr <= (others => '0');
      elsif (sys_clk = '1' and sys_clk'event) then
			hdata_sr(0) <= hdata_reg;
			hdata_sr(7 downto 1) <= hdata_sr(6 downto 0);
      end if;
   end process HDATA_SR_PROC;

	-- TFT_SDRAM_DTMG output
	tft_sdram_dtmg <= hdata_sr(7);

	-- use registers after double sampling
	hdata_sr_sig <= '1' when ((hdata_sr(6) = '1') and (hdata_sr(7) = '0')) else
	                '0';

   ----------------------------
   -- HDATA_SR_REG_PROC process
   ----------------------------
   HDATA_SR_REG_PROC: process(sys_rst, sys_clk)
   begin
      if (sys_rst = '1') then
			hdata_sr_reg <= '0';
      elsif (sys_clk = '1' and sys_clk'event) then
			hdata_sr_reg <= hdata_sr_sig;
      end if;
   end process HDATA_SR_REG_PROC;
	
	tft_row_start <= hdata_sr_reg;

   -----------------------
   -- TFT_TEST_RGB process
   -----------------------
   TFT_TEST_RGB_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
         tft_test_red   <= (others => '0');
         tft_test_green <= (others => '0');
         tft_test_blue  <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
         -- tft_test_red   <= (others => '0');
         -- tft_test_green <= (others => '0');
         -- tft_test_blue  <= (others => '0');
         -- tft_test_red   <= "00000001";
         -- tft_test_green <= (others => '1');
         -- tft_test_blue  <= "00000001";
         -- if ( (row_cnt(2) = '0') or (column_cnt(2) = '0') ) then
         -- tft_test_red   <= "01111000";
         -- tft_test_green <= "01111000";
         -- tft_test_blue  <= "01111000";
			-- else
         -- tft_test_red   <= "00111000";
         -- tft_test_green <= "01000000";
         -- tft_test_blue  <= "00001000";
			-- end if;
         -- if ( (row_cnt(2) = '0') and (column_cnt(2) = '0') ) then
         -- tft_test_red   <= "01111000";
         -- tft_test_green <= "01111000";
         -- tft_test_blue  <= "01111000";
			-- else
         -- tft_test_red   <= "00111000";
         -- tft_test_green <= "01000000";
         -- tft_test_blue  <= "00001000";
			-- end if;
			---------------
			-- TEST section
			---------------
			-- if (frame_cnt(8) = '0') then
			-- if (frame_cnt(7) = '0') then
				-- -- if ( (row_cnt(3) = '0')  or
					  -- -- (column_cnt(3) = '0') ) then
				-- if ( (column_cnt(3) = '0') ) then
					-- tft_test_red   <= (others => '1');
					-- tft_test_green <= (others => '0');
					-- tft_test_blue  <= (others => '0');
				-- else
					-- tft_test_red   <= (others => '0');
					-- tft_test_green <= (others => '0');
					-- tft_test_blue  <= (others => '1');
				-- end if;
			-- else
				-- -- if ( (row_cnt(3) = '0')  or
					  -- -- (column_cnt(3) = '0') ) then
				-- if ( (column_cnt(3) = '0') ) then
					-- tft_test_red   <= (others => '0');
					-- tft_test_green <= (others => '1');
					-- tft_test_blue  <= (others => '0');
				-- else
					-- tft_test_red   <= (others => '1');
					-- tft_test_green <= (others => '0');
					-- tft_test_blue  <= (others => '0');
				-- end if;
			-- end if;
			-- else
			-- if (frame_cnt(7) = '0') then
				-- tft_test_red   <= (column_cnt(9 downto 6) & "0000");
				-- tft_test_green <= (column_cnt(5 downto 2) & "0000");
				-- tft_test_blue  <= (column_cnt(1 downto 0) & "000000");
			-- else
				-- tft_test_red   <= (column_cnt(5 downto 2) & "0000");
				-- tft_test_green <= (column_cnt(9 downto 6) & "0000");
				-- tft_test_blue  <= (column_cnt(1 downto 0) & "000000");
			-- end if;
			-- end if;
			---------------
			-- TEST section
			---------------
         -- if ( ( (row_cnt(2 downto 0) = 0) or (row_cnt = (V_LINES - 1)) ) or
              -- ( (column_cnt(2 downto 0) = 0) or (column_cnt = (H_PIXELS - 1)) ) ) then
         ------------------------
			-- functional simulation
         ------------------------
			-- tft_test_red   <= ("000" & tft_sdram_red);
         -- tft_test_green <= ("00" & tft_sdram_green);
         -- tft_test_blue  <= ("000" & tft_sdram_blue);
         ------------------------
			-- functional simulation
         ------------------------
			tft_test_red   <= (tft_sdram_red & "000");
         tft_test_green <= (tft_sdram_green & "00");
         tft_test_blue  <= (tft_sdram_blue & "000");
      end if;
   end process TFT_TEST_RGB_PROC;

	-- tft_vsync <= vsync_reg;
	-- tft_hsync <= hsync_reg;
	-- tft_dtmg  <= hdata_reg;

   --------------------------------
   -- HV_SYNC_DATA_REG_PROC process
   --------------------------------
   -- HV_SYNC_DATA_REG_PROC: process(tft_clk)
   -- begin
      -- if (tft_clk = '1' and tft_clk'event) then
			-- vsync_reg0 <= vsync_reg;
			-- hsync_reg0 <= hsync_reg;
			-- hdata_reg0 <= hdata_reg;
      -- end if;
   -- end process HV_SYNC_DATA_REG_PROC;
   HV_SYNC_DATA_REG_PROC: process(sys_rst, tft_clk)
   begin
      if (sys_rst = '1') then
			vsync_final_sr <= (others => '0');
			hsync_final_sr <= (others => '0');
			hdata_final_sr <= (others => '0');
      elsif (tft_clk = '1' and tft_clk'event) then
			vsync_final_sr(0) <= vsync_reg;
			-- !!!!!!!!!!!!!!!!!!!!!!!!
			-- !!! SOLO PER MONITOR !!!
			-- !!!!!!!!!!!!!!!!!!!!!!!!
			--hsync_final_sr(0) <= hsync_reg;
			hsync_final_sr(0) <= debug_hsync_reg;
			-- !!!!!!!!!!!!!!!!!!!!!!!!
			-- !!! SOLO PER MONITOR !!!
			-- !!!!!!!!!!!!!!!!!!!!!!!!
			hdata_final_sr(0) <= hdata_reg;
			vsync_final_sr(15 downto 1) <= vsync_final_sr(14 downto 0);
			hsync_final_sr(15 downto 1) <= hsync_final_sr(14 downto 0);
			hdata_final_sr(15 downto 1) <= hdata_final_sr(14 downto 0);
      end if;
   end process HV_SYNC_DATA_REG_PROC;

	tft_vsync <= not vsync_final_sr(15);
	tft_hsync <= not hsync_final_sr(15);
	tft_dtmg  <= hdata_final_sr(15);
	-- tft_red   <= tft_test_red;
	-- tft_green <= tft_test_green;
	-- tft_blue  <= tft_test_blue;
	-- tft_red   <= (others => '0') when (vsync_final_sr(0) = '0' or hsync_final_sr(0) = '0') else tft_test_red;
	-- tft_green <= (others => '0') when (vsync_final_sr(0) = '0' or hsync_final_sr(0) = '0') else tft_test_green;
	-- tft_blue  <= (others => '0') when (vsync_final_sr(0) = '0' or hsync_final_sr(0) = '0') else tft_test_blue;
	tft_red   <= (others => '0') when (hdata_final_sr(15) = '0') else tft_test_red;
	tft_green <= (others => '0') when (hdata_final_sr(15) = '0') else tft_test_green;
	tft_blue  <= (others => '0') when (hdata_final_sr(15) = '0') else tft_test_blue;
--hdata_final_sr(0) <= hdata_reg
end rtl_tft_timing_ctrl;
-----------------------------------------------------------------------